#include "stdafx.h"
#include "PcmConverter.hpp"

namespace Lmt {
namespace O2Bms {
	PcmConverter::PcmConverter() {
		m_init = false;
		m_acm = NULL;
	}

	PcmConverter::~PcmConverter() {
		if (m_init == true) {
			if (m_acm != NULL) {
				::acmStreamClose( m_acm, 0 );
			}
		}
	}

	int PcmConverter::GetInputSamples() const {
		return m_inSamples;
	}

	int PcmConverter::GetInputBits() const {
		return m_inBits;
	}

	int PcmConverter::GetInputChannels() const {
		return m_inChannels;
	}

	int PcmConverter::GetOutputSamples() const {
		return m_outSamples;
	}

	int PcmConverter::GetOutputBits() const {
		return m_outBits;
	}

	int PcmConverter::GetOutputChannels() const {
		return m_outChannels;
	}

	WAVEFORMATEX PcmConverter::GetInputWaveFormat() const {
		return m_wfIn;
	}

	WAVEFORMATEX PcmConverter::GetOutputWaveFormat() const {
		return m_wfOut;
	}

	bool PcmConverter::Initialize( int inSamples, int inBits, int inChannels, int outSamples, int outBits, int outChannels ) {
		if (m_init == true) {
			return false;
		}
		if (TestSamples( inSamples ) == false || TestSamples( outSamples ) == false) {
			return false;
		}
		if (TestBits( inBits ) == false || TestBits( outBits ) == false) {
			return false;
		}
		if (TestChannels( inChannels ) == false || TestChannels( outChannels ) == false) {
			return false;
		}

		HACMSTREAM acm = NULL;
		WAVEFORMATEX wfIn;
		WAVEFORMATEX wfOut;

		m_inSamples = inSamples;
		m_inBits = inBits;
		m_inChannels = inChannels;
		m_outSamples = outSamples;
		m_outBits = outBits;
		m_outChannels = outChannels;

		//̓tH[}bg
		memset( &wfIn, 0, sizeof(wfIn) );
		wfIn.wFormatTag = WAVE_FORMAT_PCM;
		wfIn.nChannels = inChannels;
		wfIn.nSamplesPerSec = inSamples;
		wfIn.wBitsPerSample = inBits;
		wfIn.nBlockAlign = wfIn.nChannels * (wfIn.wBitsPerSample) / 8;
		wfIn.nAvgBytesPerSec = wfIn.nBlockAlign * wfIn.nSamplesPerSec;
		wfIn.cbSize = 0;
		
		//o̓tH[}bg
		memset( &wfOut, 0, sizeof(wfOut) );
		wfOut.wFormatTag = WAVE_FORMAT_PCM;
		wfOut.nChannels = outChannels;
		wfOut.nSamplesPerSec = outSamples;
		wfOut.wBitsPerSample = outBits;
		wfOut.nBlockAlign = wfOut.nChannels * (wfOut.wBitsPerSample) / 8;
		wfOut.nAvgBytesPerSec = wfOut.nBlockAlign * wfOut.nSamplesPerSec;
		wfOut.cbSize = 0;

		m_wfIn = wfIn;
		m_wfOut = wfOut;

		if (this->NeedConvert() == false) {
			m_acm = NULL;
			m_init = true;
			return true;
		}

		//ACMXg[
		if (::acmStreamOpen( &acm, NULL, &wfIn, &wfOut, NULL, NULL, NULL, 0 ) != 0) {
			goto ErrorMethod;
		}

		//I
		m_acm = acm;
		m_init = true;
		return true;

	ErrorMethod:
		if (acm != NULL) {
			::acmStreamClose( acm, 0 );
		}
		return false;
	}

	bool PcmConverter::Convert( const void *psrc, int srcLength, int *psrcUsed, void *pdest, int destLength, int *pdestUsed ) {
		if (m_init == false) {
			return false;
		}
		if (psrc == NULL || pdest == NULL) {
			return false;
		}
		if (srcLength <= 0 || destLength <= 0) {
			return false;
		}
		if (psrcUsed == NULL || pdestUsed == NULL) {
			return false;
		}

		if (this->NeedConvert() == false) {
			//ϊ̕KvȂꍇ
			int copyLength = min( srcLength, destLength );
			memcpy( pdest, psrc, copyLength );
			*psrcUsed = copyLength;
			*pdestUsed = copyLength;
			return true;
		}
		else {
			//ϊ̕Kvꍇ
			ACMSTREAMHEADER header;

			//ACMwb_[̏
			memset( &header, 0, sizeof(header) );
			header.cbStruct = sizeof(header);
			header.pbSrc = reinterpret_cast<LPBYTE>( const_cast<void*>( psrc ) );
			header.cbSrcLength = srcLength;
			header.pbDst = reinterpret_cast<LPBYTE>( pdest );
			header.cbDstLength = destLength;

			//ϊ
			if (::acmStreamPrepareHeader( m_acm, &header, 0 ) != 0) {
				return false;
			}
			if (::acmStreamConvert( m_acm, &header, ACM_STREAMCONVERTF_BLOCKALIGN ) != 0) {
				::acmStreamUnprepareHeader( m_acm, &header, 0 );
				return false;
			}

			::acmStreamUnprepareHeader( m_acm, &header, 0 );
			*psrcUsed = header.cbSrcLengthUsed;
			*pdestUsed = header.cbDstLengthUsed;
			return true;
		}
	}

	bool PcmConverter::NeedConvert() const {
		return (
			m_inSamples != m_outSamples ||
			m_inBits != m_outBits ||
			m_inChannels != m_outChannels
		);
	}

	bool PcmConverter::TestSamples( int samples ) {
		if (samples < 8000) {
			return false;
		}
		if (samples > 96000) {
			return false;
		}
		return true;
	}

	bool PcmConverter::TestBits( int bits ) {
		switch (bits) {
			case 8:
			case 16:
				return true;
			default:
				return false;
		}
	}

	bool PcmConverter::TestChannels( int channels ) {
		if (channels != 1 && channels != 2) {
			return false;
		}
		return true;
	}
}
}
